@@ -3,8 +3,9 @@ from django import template
 register = template.Library()
 
 class_converter = {
-    "textinput":"textinput textInput",
-    "fileinput":"fileinput fileUpload"
+    "textinput": "textinput textInput",
+    "fileinput": "fileinput fileUpload",
+    "passwordinput": "textinput textInput",
 }
 
 @register.filter
@@ -15,10 +16,6 @@ def is_checkbox(field):
 def with_class(field):
     class_name = field.field.widget.__class__.__name__.lower()
     class_name = class_converter.get(class_name, class_name)
-    if "class" in field.field.widget.attrs:
-        field.field.widget.attrs['class'] += " %s" % class_name
-    else:
-        field.field.widget.attrs['class'] = class_name
-    return unicode(field)
-
+    field.field.widget.attrs['class'] = field.css_classes(extra_classes=class_name)
 
+    return field
